#### Libraries ####
library(minpack.lm)
library(optimx)
library(forecast)
library(PtProcess)
library(tseries)
library(formattable) 
library(htmltools)
library(webshot)

#### Tests of functions to get the exceedance ####
# These functions work elementwise on the matrices to build
# Building of A_kj
building_matrices <- function(X, level)
{
  # initialisation
  N <- length(x = X)
  init <- rep(0, times = (N-1)*(N-1))
  A = matrix(data = init ,nrow =  N-1, ncol = N-1) 
  # N - 1 because k starts at 2
  
  # Calculations
  for(k in 2:N)
  {
    for(j in k:N)
    {
      if(max(X[(j-k+1):(j-1)]) <= level)
      {
        if(X[j] > level)
        {
          A[k-1,j-1] = 1
        }
      }
    }
  }
  return(A)
}


# Just calculation of the expected values 
sum_peak <- function(X, level)
{
  # initialisation
  N <- length(x = X)
  a = rep(0,N-1)
  # Calculations
  for(k in 2:N)
  {
    for(j in k:N)
    {
      if(max(X[(j-k+1):(j-1)]) <= level)
      {
        if(X[j] > level)
        {
          a[k-1] <- a[k-1] + 1 #sum of a_kj for k fixed
        }
      }
    }
  }
  return(a)
}

# This function works on the data directly
sum_validate_data <- function(X, level)
{
  N <- length(X)
  a <- rep(0, N-1)
  # Calculation of a and b 
  index <- which(X > level)
  l <- length(index)
  if(l==0)
  {
    return(a)
  }
  # firt index
  aux <- index[1] 
  if(aux > 0)
  {
    a[1:(aux)] <- a[1:(aux)] + rep(1,aux)
  }
  if(l >1)
  {
    for (i in 1:(l-1))
    {
      
      aux <- index[i+1] - index[i] - 1
      if(aux > 1)
      {
        a[1:(aux)] <- a[1:(aux)] + rep(1,aux)
      }
    }
  }
  return(a)
}

test_exceedance_funct <- function()
{
    l <- 10
    t_1 <- rep(0, l)
    t_2 <- rep(0, l)
    t_3 <- rep(0, l)
    aux <- rep(0, l)
    level <- 2
    for(l in 1:l){
      X <- rpareto(l*100, 3, 1)
      t <- Sys.time()
      A <- building_matrices(X,level)
      aux <- sum(A)
      t_1[l] <- Sys.time()-t
      t <- Sys.time()
      a <- sum_peak(X,level)
      t_2[l] <- Sys.time()-t
      t <- Sys.time()
      a <- sum_validate_data(X,level)
      t_3[l] <- Sys.time()-t
    }
    x <- seq(100, l*100, 100)
    plot(x = x, y = t_1, type = "l", col = "blue", 
         xlab = "Length of X", ylab = "Time in second")
    lines(x = x, y = t_2, type = "l", col = "green" )
    lines(x = x, y = t_3, type = "l", col = "red" )
    legend("topleft", legend = c("building_matrices", 
    "sum_peak", "sum_validate_data"), 
    col=c("blue", "green", "red"), lty = c(1,1,1))
}

#### Tests on CI functions and calculation of ACER function ####
# Calculation of Epsilon
epsilon_app <- function(X,aux_sum, k)
  {
    # Here we just divide the number of exceedance 
    # by approximately the number of points to get
    # a probability
    epsilon <- aux_sum[k]/(length(X)+1-k+1)
    return(epsilon)
    # }
  }


# Confidence interval

# Block method  
CI_block <- function(X, level, k)
{
  
  # First we test the seasonality best chance to get 
  # a block structure
  model <- auto.arima(X, D=1) 
  # to force the auto.arima to find a seasonability
  b <- model$arma
  s <- b[5] ### s get the seasonality
  s <- 3
  aux_mean <- rep(0, s-1)
  # we have to share the data into blocks of 
  # length s and work on each blocks
  N <- length(X)
  R <- floor(N/s)
  list_aux <- list()
  x_aux <- rep(0, N)
  for(i in 1:R)
  {
    x <- X[((i-1)*s+1) : (i*s)]
    x_aux <- sum_validate_data(x, level)
    list_aux[[i]] <- epsilon_app(X, x_aux, k)
  }
  
  # last block
  
  x <- X[(R*s+1) : N]
  x_aux <- sum_validate_data(x, level)
  list_aux[[R+1]] <- epsilon_app(x_aux, length(x_aux), k)
  
  eps_aux <- data.frame(t(sapply(list_aux,c)))
  # Calculation of s_k
  std_dev <- rep(0, s-1)
  for(k in 1:(s-1))
  {
    l_aux <- list_aux[[k]]
    aux <- unlist(l_aux)
    aux[is.na(aux)] <- 0
    aux_mean[k] <- mean(aux)
    aux2 <- aux - rep(aux_mean[k], length(aux))
    std_dev[k] <- (sum(aux2*aux2))/(R-1)
  }
  
  # Creation of the confidence interval
  C_plus <- aux_mean + 1.96*sqrt(std_dev)/sqrt(R)
  C_minus <- aux_mean - 1.96*sqrt(std_dev)/sqrt(R)
  C <- data.frame(C_plus, C_minus)
  return(list("CI" = C, "epsilon" = eps_aux))
}

CI_Poisson <- function(X, level, k)
{
  #First we calculate the value of epsilon
  x <- sum_validate_data(X, level)
  N <- length(X)
  eps_aux <- epsilon_app(X,x, k)
  
  # Then we apply the classical formula of a CI for a Poisson 
  #distribution
  if(eps_aux == 0){
    C_plus <- 1.96*sqrt((N-k+1))
    C_minus <- -1.96*sqrt((N-k+1))
  }
  else{
    C_plus <- eps_aux*(1 + 1.96/sqrt((N-k+1)*eps_aux))
    C_minus <- eps_aux*(1 - 1.96/sqrt((N-k+1)*eps_aux))
  }
  
  
  C <- data.frame(C_plus, C_minus)
  return(list("CI" = C, "epsilon" = eps_aux))
}

CI_boot <- function(X, level, k, nb)
{
  nb_sampling <- nb
  L <- rep(0, nb_sampling)
  for(j in 1:nb_sampling){
    #We calculate the value of epsilon for sample based on 
    # the initial data set.
    x <- sum_validate_data(sample(X, replace = T), level)
    L[j] <- epsilon_app(X,x, k)
  }
  
  # we take the alpha % confidence interval
  aux <- sort(L)
  C_minus <- aux[floor(0.025*length(aux))]
  C_plus <- aux[floor(0.975*length(aux))+1]
  
  C <- list(C_plus, C_minus)
  x <- sum_validate_data(X, level)
  eps_aux <- epsilon_app(X,x, k)
  
  return(list("CI" = C, "epsilon" = eps_aux))
}


level <- 2
k <- 6
nb_samp <- 50
len <- 20
t_1 <- rep(0, len)
t_2 <- rep(0, len)
e_k_2 <- rep(0, len)
a_2 <- rep(0, len)
t_3 <- rep(0, len)
e_k_3 <- rep(0, len)
a_3 <- rep(0, len)
level <- 2

test_CI_time <- function(){
    # TIME
    for(l in 1:len){
      X <- rpareto(l*100, 3, 1)
      t <- Sys.time()
      b <- CI_block(X, level, k)
      t_1[l] <- Sys.time()-t
      t <- Sys.time()
      c <- CI_Poisson(X, level, k)
      t_2[l] <- Sys.time()-t
      t <- Sys.time()
      d <- CI_boot(X, level,k, nb_samp)
      t_3[l] <- Sys.time()-t
    }
    x <- seq(100, len*100, 100)


    plot(x = x, y = t_1, type = "l", col = "blue", 
        xlab = "Length of X", ylab = "time in second")
    lines(x = x, y = 25*t_2, type = "l", col = "green" )
    lines(x = x, y = t_3, type = "l", col = "red" )
    legend("topleft", legend = c("CI_block", "25*CI_Poisson",
    "CI_boot"), col=c("blue", "green", "red"), lty = c(1,1,1))
}

test_CI_acc <- function(){
    # ACCURACY vs length
    for(l in 1:len){
      X <- rpareto(l*100, 3, 1)
      t <- Sys.time()
      a <- CI_Poisson(X, level, k)
      e_k_2[l] <- (a$epsilon)[k]
      diff_2 <- (a$CI)[1]- (a$CI)[2]
      a_2[l] <- sqrt(sum(diff_2*diff_2))
      t_2[l] <- Sys.time()-t
      t <- Sys.time()
      b <- CI_boot(X, level,k, nb_samp)
      e_k_3[l] <- (b$epsilon)[k]
      diff_3 <- (b$CI)[1]- (b$CI)[2]
      a_3[l] <- sqrt(sum(diff_3*diff_3))
      t_3[l] <- Sys.time()-t
    }
    x <- seq(100, len*100, 100)
    plot(x = x, y = a_2, type = "l", col = "blue", 
         xlab = "Length of X", ylab = "Accuracy")
    lines(x = x, y = 20*a_3, type = "l", col = "red" )
    legend("topleft", legend = c("CI_Poisson", "20*CI_boot"), 
           col=c("blue", "red"), lty = c(1,1,1))

    plot(x = e_k_2, y = e_k_3, col = "blue", 
    xlab = "Epsilon by CI_Poisson", ylab = "Epsilon by CI_boot")
    lines(e_k_2,e_k_2, col = "red" )
}

   
test_CI_boot_nb_sample <- function(){ 
    # TIME and ACCURACY vs nb_sample
    for(l in 1:len){
      X <- rpareto(2000, 3, 1)
      t <- Sys.time()
      b <- CI_boot(X, level,k, l*10)
      e_k_3[l] <- (b$epsilon)[k]
      diff_3 <- (b$CI)[1]- (b$CI)[2]
      a_3[l] <- sqrt(sum(diff_3*diff_3))
      t_3[l] <- Sys.time()-t
    }
    x <- seq(10, len*10, 10)
    par(mar=c(4,4,3,5))
    plot(x = x, y = t_3, type = "l", col = "blue", 
         xlab = "Number of samplings", ylab = "Time")

    par(new=T)
    plot(x = x, y = a_3, type = "l", col = "red", axes=F, 
         xlab = "", ylab ="")
    mtext("Length of the CI",side=4, line = 2.5)
    axis(4)
    legend(x="topleft",legend=c("Time","Length of the CI"),
           col=c("blue","red"),lty = c(1,1))
}

#### Tests optimization algorithms ####
# Detection of the tail marker eta1
eta_detect <- function(X, k, alpha)
{
  nb_values <- 200
  pas <- (max(X)-min(X))/nb_values
  test_level <- seq(min(X),max(X),pas)
  i <- 1
  k_i <- k
  aux_end <- rep(0,k_i)
  for(level in test_level)
  {
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    if(aux1>0){
      aux_end[i] <- (aux1)

    }
    else{
      aux_end[i] <- 0
    }
    i = i+1
  }
  m <- max(aux_end)
  ind <- max(which(aux_end == m))
  v <- aux_end[ind:length(aux_end)]
  mu1 <- test_level[ind]
  return(mu1)
}

# Start research model

ACER_Gumble_1 <- function(X, k, alpha)
{
  k_i <- k
  mu1 <- eta_detect(X, k)
  muf <- max(X)
  nb_points <- 100
  pas <- (muf - mu1)/(nb_points)
  test_level <- seq(mu1, muf, pas)

  aux <- CI_Poisson(X,mean(X),k_i)
  aux1 <- aux$epsilon
  aux2 <- aux$CI
  aux_eps <- aux1
  print(aux)
  aux_w <- 1/(log(aux2[1]-aux2[2]))^2
  for(level in test_level)
  {
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    aux2 <- aux$CI
    v <- 1/(log(aux2[1] -aux2[2])^2)

    aux_eps <- c(aux_eps, aux1)
    aux_w <- c(aux_w, v)

  }
  aux_w <- aux_w[-1]
  aux_eps <- aux_eps[-1]



  # create the functions to optimize
  eps <- aux_eps[aux_eps != 0]
  w <- aux_w[aux_eps != 0]
  test_level <- test_level[aux_eps != 0]
  level <- test_level
  ACER_funct_value <- function(param){

    q <- param[1]
    a <- param[2]
    b <- param[3]
    c <- param[4]
    result <- 0
    result <- sum(w*abs((log(eps) - log(q) + a*(test_level - b)^c))^2)

    return(result)
  }


  U <- matrix(data = c(1,0,0,0, 0, 1, 0, 0, 0, 0 ,1, 0, 0, 0 ,-1, 0, 
        0, 0, 0, 1, 0, 0, 0, -1), nrow = 6, ncol = 4, byrow = TRUE)
  m <- min(X)
  M <- max(X)
  C <- c(0 ,0,  m, -mu1, 0, -5)

  minit <- (m+mu1)/2
  print(minit)
  print(m)
  print(mu1)
  o <- constrOptim(c(1,1, minit, 2), ACER_funct_value, NULL, U, C, 
                   outer.eps = 0.0001 )  
  param <- o$par
  q <- param[1]
  a <- param[2]
  b <- param[3]
  c <- param[4]

  VaR <- b+(1/a*(abs(log(alpha/q))))^(1/c)

  return(list(q = q, a = a, b = b, c = c, VaR = VaR, 
              tl = test_level, eps = eps))
}

ACER_Gumble_2 <- function(X, k, alpha)
{
  k_i <- k
  mu1 <- eta_detect(X, k)
  muf <- max(X)
  nb_points <- 100
  pas <- (muf - mu1)/(nb_points)
  test_level <- seq(mu1, muf, pas)

  aux <- CI_Poisson(X,mean(X),k_i)
  aux1 <- aux$epsilon
  aux2 <- aux$CI
  aux_eps <- aux1
  print(aux)
  aux_w <- 1/(log(aux2[1]-aux2[2]))^2
  for(level in test_level)
  {
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    aux2 <- aux$CI
    v <- 1/(log(aux2[1] -aux2[2])^2)

    aux_eps <- c(aux_eps, aux1)
    aux_w <- c(aux_w, v)

  }
  aux_w <- aux_w[-1]
  aux_eps <- aux_eps[-1]



  # create the functions to optimize
  eps <- aux_eps[aux_eps != 0]
  w <- aux_w[aux_eps != 0]
  test_level <- test_level[aux_eps != 0]
  level <- test_level
  ACER_funct_vect <- function(param){
    q <- param[1]
    a <- param[2]
    b <- param[3]
    c <- param[4]
    result <- rep(0, length(w))
    for(i in 1:length(w)){
      result[i] <- as.double(sqrt(w[i])*abs(log(eps[i]) - log(q) 
                            + a*abs(test_level[i] - b)^c))
    }
    return(result)
  }

  m <- min(X)
  M <- max(X)

  minit <- (m+mu1)/2
  o <- nls.lm(c(1,1, minit, 1), lower = c(0, 0, 0.99*m, 0), 
              upper = c(Inf, Inf, 1.01*mu1, 5),ACER_funct_vect,  
          jac = NULL,
          nls.lm.control(ftol = 0.0001,maxiter = 1000)) 
  param <- o$par
  q <- param[1]
  a <- param[2]
  b <- param[3]
  c <- param[4]
  VaR <- b+(1/a*(-log(alpha/q)))^(1/c)
  return(list(q = q, a = a, b = b, c = c, VaR = VaR, 
              tl = test_level, eps = eps))
}

ACER_Gumble_3 <- function(X, k, alpha)
{
  k_i <- k
  mu1 <- eta_detect(X, k)
  muf <- max(X)
  nb_points <- 100
  pas <- (muf - mu1)/(nb_points)
  test_level <- seq(mu1, muf, pas)

  aux <- CI_Poisson(X,mean(X),k_i)
  aux1 <- aux$epsilon
  aux2 <- aux$CI
  aux_eps <- aux1
  print(aux)
  aux_w <- 1/(log(aux2[1]-aux2[2]))^2
  for(level in test_level)
  {
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    aux2 <- aux$CI
    v <- 1/(log(aux2[1] -aux2[2])^2)

    aux_eps <- c(aux_eps, aux1)
    aux_w <- c(aux_w, v)

  }
  aux_w <- aux_w[-1]
  aux_eps <- aux_eps[-1]



  # create the functions to optimize
  eps <- aux_eps[aux_eps != 0]
  w <- aux_w[aux_eps != 0]
  test_level <- test_level[aux_eps != 0]
  level <- test_level

  y <- log(eps)
  y_m <- sum(w*y) / sum(w)

  ACER_sim <- function(param){
    b <- param[1]
    c <- param[2]

    x <- (level - b)^c
    x_m <- sum(w*x)/sum(w)
    a <- -(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
    q <- exp(y_m + a*x_m)
    result <- sum(w*(y-log(q)+a*x)^2)
    return(as.double(result))
  }

  #
  #
  # ## we need to find b, c
  U <- matrix(data = c( 1, 0,-1, 0, 0, 1, 0, -1), nrow = 4, 
              ncol = 2, byrow = TRUE)
  m <- min(X)
  C <- c(0.99*m, -1.01*mu1, 0, -5)
  minit <- (mu1+m)/2
  o <- constrOptim(c(minit, 1), ACER_sim, NULL,
                   method =  "Nelder-Mead", 
                   U, C, outer.eps = 1e-07)

  param <- o$par
  b <- param[1]
  c <- param[2]
  x <- (level - b)^c
  x_m <- sum(w*x)/sum(w)
  a <- abs(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
  q <- exp(y_m + a*x_m)


  VaR <- b+(1/a*(-log(alpha/q)))^(1/c)

  return(list(q = q, a = a, b = b, c = c, VaR = VaR, 
              tl = test_level, eps = eps))
}

ACER_Gumble_4 <- function(X, k, alpha)
{
  k_i <- k
  mu1 <- eta_detect(X, k)
  muf <- max(X)
  nb_points <- 100
  pas <- (muf - mu1)/(nb_points)
  test_level <- seq(mu1, muf, pas)

  aux <- CI_Poisson(X,mean(X),k_i)
  aux1 <- aux$epsilon
  aux2 <- aux$CI
  aux_eps <- aux1
  print(aux)
  aux_w <- 1/(log(aux2[1]-aux2[2]))^2
  for(level in test_level)
  {
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    aux2 <- aux$CI
    v <- 1/(log(aux2[1] -aux2[2])^2)

    aux_eps <- c(aux_eps, aux1)
    aux_w <- c(aux_w, v)

  }
  aux_w <- aux_w[-1]
  aux_eps <- aux_eps[-1]



  # create the functions to optimize
  eps <- aux_eps[aux_eps != 0]
  w <- aux_w[aux_eps != 0]
  test_level <- test_level[aux_eps != 0]
  level <- test_level
  y <- log(eps)
  y_m <- sum(w*y) / sum(w)



  ACER_sim_vect <- function(param){
    b <- param[1]
    c <- param[2]

    x <- abs(level - b)^c
    x_m <- sum(w*x)/sum(w)
    a <- abs((sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2))))
    q <- exp(y_m + a*x_m)
    result <- as.double(w*(y-q+a*x)^2)
    return(result)
  }


  # ## we need to find b, c
  m <- min(X)
  minit <- (mu1+m)/2

  o <- nls.lm(c(minit, 1), lower = c(m, 1), upper = c(mu1, 5),
      ACER_sim_vect,  jac = NULL, 
      nls.lm.control(ftol = 0.001,  maxiter = 1000))

  param <- o$par
  #Now, we need to find a, q_ln
  b <- param[1]
  c <- param[2]
  x <- abs(level - b)^c
  x_m <- sum(w*x)/sum(w)
  a <- abs(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
  q <- exp(y_m + a*x_m)


  VaR <- b+(1/a*(abs(log(alpha/q))))^(1/c)

  return(list(q = q, a = a, b = b, c = c, VaR = VaR))
}



X <- rpareto(2000, 3, 1)

## error optimization algo
test_error_opti_alg <- function(X){
    v <- ACER_Gumble_1(X,20, 0.05)
    ep <- v$eps
    ep <- log(ep)
    x <- v$tl
    q <- v$q
    a <- v$a
    b <- v$b
    c <- v$c
    a_1 <- log(q) -a*abs(x-b)^c
    e_1 <- sqrt((ep-a_1)*(ep-a_1))
    plot(x=x[x>2], y=0.5*e_1[x>2], type = "l", xlab = "Level", 
         ylab = "Error", ylim = c(0,0.7))

    v <- ACER_Gumble_2(X, 20, 0.05)
    q <- v$q
    a <- v$a
    b <- v$b
    c <- v$c
    a_1 <- log(q) -a*abs(x-b)^c
    e_1 <- sqrt((ep-a_1)*(ep-a_1))
    lines(x[x>2], e_1[x>2], type = 'l', col = "blue")

    v <- ACER_Gumble_3(X, 20, 0.05)
    q <- v$q
    a <- v$a
    b <- v$b
    c <- v$c
    a_1 <- log(q) -a*abs(x-b)^c
    e_1 <- sqrt((ep-a_1)*(ep-a_1))
    lines(x[x>2], e_1[x>2], type = 'l', col = "green")

    v <- ACER_Gumble_4(X, 20, 0.05)
    q <- v$q
    a <- v$a
    b <- v$b
    c <- v$c
    a_1 <- log(q) -a*abs(x-b)^c
    e_1 <- sqrt((ep-a_1)*(ep-a_1))
    lines(x[x>2], e_1[x>2], type = 'l', col = "red")

    legend(x="topright",
      c("0.5*Lin_Comp","LM_Comp", "Lin_Simpl","LM_Simpl"),
      col=c("black","blue", "green","red"),lty = c(1,1,1,1))
}

test_time_opti_alg <- function(){
    # execution time algo
    len <- 20
    t_1 <- rep(0,len)
    t_2 <- rep(0,len)
    t_3 <- rep(0,len)
    t_4 <- rep(0,len)

    for(l in 1:len){
      X <- rpareto(1000+100*l, 3, 1)
      t <- Sys.time()
      v <- ACER_Gumble_1(X,8,0.05)
      t_1[l] <- Sys.time()-t
      t <- Sys.time()
      v <- ACER_Gumble_2(X,8,0.05)
      t_2[l] <- Sys.time()-t
      t <- Sys.time()
      v <- ACER_Gumble_3(X,8,0.05)
      t_3[l] <- Sys.time()-t
      t <- Sys.time()
      v <- ACER_Gumble_4(X,8,0.05)
      t_4[l] <- Sys.time()-t
    }
    x <- seq(1100, 3000, 100)
    plot(x, t_1, col = "red", type = "l", ylab = "Time", 
         xlab = "Number of points")
    lines(x, t_2, type = "l",col= "blue")
    lines(x, t_3,type = "l", col= "green")
    lines(x, t_4, type = "l",col = "black")
    legend(x="topleft",
           c("Lin_Comp","LM_Comp", "Lin_Simpl", "LM_Simpl"),
           col=c("black","blue", "green","red"),lty = c(1,1,1,1))
}





## test_LM_algo_nbPoints
test_nb_points <- function(){
    #here we will use LM_simpl and LM_Compl
    len <- 20
    t_1 <- rep(0,len)
    t_2 <- rep(0,len)
    t_3 <- rep(0,len)
    t_4 <- rep(0,len)

    for(l in 1:len){
      X <- rpareto(500+200*l, 3, 1)
      t <- Sys.time()
      v <- ACER_Gumble_1(X,8,0.05)
      t_1[l] <- Sys.time()-t
      t <- Sys.time()
      v <- ACER_Gumble_4(X,8,0.05)
      t_2[l] <- Sys.time()-t
    }
    x <- seq(700, 4500, 200)
    plot(x, t_1, col = "red", type = "l", ylab = "Time", 
         xlab = "Number of points")
    lines(x, t_2, type = "l",col = "black")
    legend(x="topleft",legend=c("Lin_Simpl", "LM_Simpl"),
           col=c("black","red"),lty = c(1,1))
}

#Now we choose LM_simpl (ACER_gumbel_4)
test_nb_pts_time_acc <- function(){    
  #we want to test the influence of the number of points on 
  # the time and the accuracy with VaR
    len <- 10
    t <- rep(0,len)
    ac <- rep(0,len)
    alpha <- 0.001

    # X <- rpareto(1000, 3, 1)
    # v <- ACER_Gumble_4(X,3,alpha)
    # print(v$VaR)

    for(l in 1:len){

      for(i in 1:3){
        X <- rpareto(300+300*l, 3, 1)
        t_aux <- Sys.time()
        v <- ACER_Gumble_4(X,3,alpha)
        print(v)
        t[l] <- t[l] + Sys.time()-t_aux
        ac[l] <- ac[l] + v$VaR
      }
      ac[l] <- ac[l]/3
      t[l] <- t[l]/3
    }
    ac <- abs(ac - 10)/10

    x <- seq(300, len*300, 300)
    par(mar=c(4,4,3,5))
    plot(x = x, y = ac, type = "l", col = "blue", 
         xlab = "Number of points", ylab = "Accuracy")

    par(new=T)
    plot(x = x, y = t, type = "l", col = "red", axes=F, 
         xlab = "", ylab ="")
    mtext("Time",side=4, line = 2.5)
    axis(4)
    legend(x="topleft",legend=c("Accuracy","Time"),
           col=c("blue","red"), lty = c(1,1))
}


## test precision mu1
test_accur_eta <- function(){
      eta_detect_test <- function(X,n){
          nb_values <- 200
          pas <- (max(X)-min(X))/nb_values
          test_level <- seq(min(X),max(X),pas)
          i <- 1
          k_i <- k
          aux_end <- rep(0,k_i)
          for(level in test_level)
          {
            aux <- CI_Poisson(X, level, k_i)
            aux1 <- aux$epsilon
            if(aux1>0){
              aux_end[i] <- (aux1)
      
            }
            else{
              aux_end[i] <- 0
            }
            i = i+1
          }
          m <- max(aux_end)
          ind <- max(which(aux_end == m))
          v <- aux_end[ind:length(aux_end)]
          mu1 <- test_level[ind+n]
      
      
        return(mu1)
      }
      
      ACER_Gumble_4_eta1 <- function(X, k, alpha,n)
      {
        k_i <- k
        mu1 <- eta_detect_test(X,n)
        muf <- max(X)
        nb_points <- 100
        pas <- (muf - mu1)/(nb_points)
        test_level <- seq(mu1, muf, pas)
      
        aux <- CI_Poisson(X,mean(X),k_i)
        aux1 <- aux$epsilon
        aux2 <- aux$CI
        aux_eps <- aux1
        print(aux)
        aux_w <- 1/(log(aux2[1]-aux2[2]))^2
        for(level in test_level)
        {
          aux <- CI_Poisson(X, level, k_i)
          aux1 <- aux$epsilon
          aux2 <- aux$CI
          v <- 1/(log(aux2[1] -aux2[2])^2)
      
          aux_eps <- c(aux_eps, aux1)
          aux_w <- c(aux_w, v)
      
        }
        aux_w <- aux_w[-1]
        aux_eps <- aux_eps[-1]
      
      
      
        # create the functions to optimize
        eps <- aux_eps[aux_eps != 0]
        w <- aux_w[aux_eps != 0]
        test_level <- test_level[aux_eps != 0]
        level <- test_level
        y <- log(eps)
        y_m <- sum(w*y) / sum(w)
      
      
      
        ACER_sim_vect <- function(param){
          b <- param[1]
          c <- param[2]
      
          x <- abs(level - b)^c
          x_m <- sum(w*x)/sum(w)
          a <- abs((sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2))))
          q <- exp(y_m + a*x_m)
          result <- as.double(w*(y-q+a*x)^2)
          return(result)
        }
      
      
        # ## we need to find b, c
        m <- min(X)
        minit <- (mu1+m)/2
      
        o <- nls.lm(c(minit, 1), lower = c(m, 1),
                    upper = c(mu1, 5),ACER_sim_vect,  jac = NULL, 
                    nls.lm.control(ftol = 0.001, maxiter = 1000))  
      
        param <- o$par
        #Now, we need to find a, q_ln
        b <- param[1]
        c <- param[2]
        x <- abs(level - b)^c
        x_m <- sum(w*x)/sum(w)
        a <- abs(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
        q <- exp(y_m + a*x_m)
      
      
        VaR <- b+(1/a*(abs(log(alpha/q))))^(1/c)
      
        return(list(q = q, a = a, b = b, c = c, VaR = VaR))
      }
      
      len <- 10
      ac <- rep(0,len)
      alpha <- 0.01
      
      # X <- rpareto(1000, 3, 1)
      # v <- ACER_Gumble_4(X,3,alpha)
      # print(v$VaR)
      X <- rpareto(1500, 3, 1)
      for(l in 1:len){
      
        for(i in 1:1){
      
          v <- ACER_Gumble_4_eta1(X,3,alpha,l)
          print(v)
          ac[l] <- ac[l] + v$VaR
        }
        ac[l] <- ac[l]/1
      }
      ac <- abs(ac - 10)/10
      
      x <- seq(1, len, 1)
      plot(x = x, y = ac, type = "l", col = "blue", xlab = "n", 
           ylab = "Accuracy")
}


## test nb levels
test_nb_lvl <- function(){
    ACER_Gumble_4_lev <- function(X, k, alpha, nb_level)
    {
        k_i <- k
        mu1 <- eta_detect(X,k)
        muf <- max(X)
        nb_points <- nb_level
        pas <- (muf - mu1)/(nb_points)
        test_level <- seq(mu1, muf, pas)
    
        aux <- CI_Poisson(X,mean(X),k_i)
        aux1 <- aux$epsilon
        aux2 <- aux$CI
        aux_eps <- aux1
        print(aux)
        aux_w <- 1/(log(aux2[1]-aux2[2]))^2
        for(level in test_level)
        {
          aux <- CI_Poisson(X, level, k_i)
          aux1 <- aux$epsilon
          aux2 <- aux$CI
          v <- 1/(log(aux2[1] -aux2[2])^2)
    
          aux_eps <- c(aux_eps, aux1)
          aux_w <- c(aux_w, v)
    
        }
        aux_w <- aux_w[-1]
        aux_eps <- aux_eps[-1]
    
    
    
        # create the functions to optimize
        eps <- aux_eps[aux_eps != 0]
        w <- aux_w[aux_eps != 0]
        test_level <- test_level[aux_eps != 0]
        level <- test_level
        y <- log(eps)
        y_m <- sum(w*y) / sum(w)
    
    
    
        ACER_sim_vect <- function(param){
          b <- param[1]
          c <- param[2]
    
          x <- abs(level - b)^c
          x_m <- sum(w*x)/sum(w)
          a <- abs((sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2))))
          q <- exp(y_m + a*x_m)
          result <- as.double(w*(y-q+a*x)^2)
          return(result)
        }
    
    
        # ## we need to find b, c
        m <- min(X)
        minit <- (mu1+m)/2
    
        o <- nls.lm(c(minit, 1), lower = c(m, 1), 
             upper = c(mu1, 5), ACER_sim_vect,  jac = NULL, 
             nls.lm.control(ftol = 0.001, maxiter = 1000))
    
        param <- o$par
        #Now, we need to find a, q_ln
        b <- param[1]
        c <- param[2]
        x <- abs(level - b)^c
        x_m <- sum(w*x)/sum(w)
        a <- abs(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
        q <- exp(y_m + a*x_m)
    
    
        VaR <- b+(1/a*(abs(log(alpha/q))))^(1/c)
    
        return(list(q = q, a = a, b = b, c = c, VaR = VaR))
    }
    
    len <- 20
    t <- rep(0, len)
    ac <- rep(0,len)
    X <- rpareto(2000, 3,1)
    k <- 8
    alpha <- 0.01
    for(i in 1:len){
      t_aux <- Sys.time()
      v <- ACER_Gumble_4_lev(X,k,alpha, 50*len)
      t[i] <- Sys.time() - t_aux
      ac[i] <- abs(v$VaR - 10)/10
    }
    x <- seq(50, len*50, 50)
    par(mar=c(4,4,3,5))
    plot(x = x, y = ac, type = "l", col = "blue", 
         xlab = "Number of levels", ylab = "Accuracy")
    
    par(new=T)
    plot(x = x, y = t, type = "l", col = "red", axes=F, 
         xlab = "", ylab ="")
    mtext("Time",side=4, line = 2.5)
    axis(4)
    legend(x="topleft",legend=c("Accuracy","Time"),
           col=c("blue","red"), lty = c(1,1))
}



#### 10% test ####
test_ten_percentage <- function(){
      ## Calulation of k
    
      #This function f_k will count the number of different values
      # of the ACER functions between eta_1 and eta_f
      f_k<- function(X,k,eta1, etaf, alpha)
      {
    
        # aux <- CI_boot(X,mean(X),k, 50, alpha)
        aux <- CI_Poisson(X, mean(X),k)
        aux1 <- aux$epsilon
        aux_eps <- aux1
        nb_points <- 100
        pas <- abs(etaf - eta1)/(nb_points)
    
        test_level <- seq(eta1, etaf, pas)
        eps <- rep(0,nb_points)
        for(level in test_level)
        {
          # aux <- CI_boot(X, level,k, 100)
          aux <- CI_Poisson(X, level,k)
    
          aux1 <- aux$epsilon
          eps <- c(eps, aux1)
        }
    
        v_dif <- length(unique(eps[eps!=0]))
    
        return(v_dif)
      }
    
      #This function detect_k will choose a k to
      # get a value of k as small as possible without
      # having a lack of precision with long sequences
      #of constant values
      detect_k <- function(X, eta1, etaf, alpha)
      {
        N <- length(X)
        M <- 105
        m <- 5
        bi <- f_k(X,m, eta1, etaf, alpha)
    
        q <- floor(0.8*bi)
        while((M-m) > 5){
          m_aux <- floor((M+m)/2)
          v <- f_k(X,m_aux, eta1, etaf, alpha)
          if(v < q){
            M <- m_aux
          }
          else{
            m <- m_aux
          }
        }
        return(m)
      }
    
    
    
      ## Calculation of etaf1, eta_f
      #eta1 will represent the lowest level and eta_f the
      #highest of our study
      eta_detect <- function(X, k, alpha)
      {
        nb_values <- 200
        pas <- (max(X)-min(X))/nb_values
        test_level <- seq(min(X),max(X),pas)
        i <- 1
        k_i <- k
        aux_end <- rep(0,k_i)
        for(level in test_level)
        {
          # aux <- CI_boot(X,level,k_i,100, alpha)
          aux <- CI_Poisson(X, level, k_i)
          aux1 <- aux$epsilon
          if(aux1>0){
            aux_end[i] <- (aux1)
    
          }
          else{
            aux_end[i] <- 0
          }
          i = i+1
        }
        m <- max(aux_end)
        ind <- max(which(aux_end == m))
        v <- aux_end[ind:length(aux_end)]
        mu1 <- test_level[ind]
        w <- diff(log(v))
        l <- which(w != 0)
        r <- diff(l)
    
        m <- min(which(r >8))
        p <- l[m]
        muf <- test_level[p]
    
        return(c(mu1, muf))
      }
    
    
      ## ACER method
      ACER_General <- function(X, alpha)
      {
        #First we fixe k at a possible value
        # and we find eta1 and eta_f. These two values
        #do not vary too much with k
        eta <- eta_detect(X, 3, alpha)
        eta1 <- eta[1]
        etaf <- eta[2]
    
        #With eta1 and eta_f, we find k.
        k <- detect_k(X, eta1, etaf, alpha)
    
        #We calculate then the true values of eta1 and eta_f
        eta <- eta_detect(X, k, alpha)
        eta1 <- eta[1]
        etaf <- eta[2]
    
        nb_points <- 200
    
        pas <- (etaf-eta1)/(nb_points)
        test_level <- seq(eta1, etaf, pas)
        aux_eps <- 0
        aux_w <- 0
        for(level in test_level)
        {
          # aux <- CI_boot(X, level,k, 100)
          aux <- CI_Poisson(X, level, k)
          aux1 <- aux$epsilon
          aux2 <- unlist(aux$CI)
          v <- 1/(log(abs(aux2[1])) -log(abs(aux2[2])))^2
    
          aux_eps <- c(aux_eps, aux1)
          aux_w <- c(aux_w, v)
    
        }
    
        eps <- aux_eps[2:length(aux_eps)]
        w <- aux_w[2:length(aux_w)]
    
        level <- test_level
    
        y <- log(eps)
        # plot(w)
        y_m <- sum(w*y) / sum(w)
    
        ACER_sim_vect <- function(param){
          #we are creating now the function for the given k
          b <- param[2]
          c <- param[3]
          a_t <- param[1]
          x <- log(abs(1 + a_t*abs(level - b)^c))
          x_m <- sum(w*x)/sum(w)
          gam <- -(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
          q <- exp(y_m + gam*x_m)
          result <- rep(0, length(w))
          for(i in 1:length(w)){
            result[i] <- as.double(sqrt(w[i])*abs(y[i] - log(q)
                        + gam*log(abs(1 + a_t*abs(level[i] - b)^c))))
          }
          return(unlist(result))
        }
        #
        #
        m <- min(X)
        # we need to find b, c
        minit <- (m+eta1)/2
    
    
        o <- nls.lm(c(1,minit, 1), lower = c(0,m, 0), 
                    upper = c(Inf,eta1, 5), 
                    ACER_sim_vect,  jac = NULL, 
                    nls.lm.control(ftol = 0.001, maxiter = 1000))
    
    
        param <- o$par
        #Now, we need to find a, q_ln
        a_t <- param[1]
        b <- param[2]
        c <- param[3]
        x <- log((1 + a_t*abs(level - b)^c))
        x_m <- sum(w*x)/sum(w)
    
        gam <- -(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
    
        q <- exp((y_m + gam*x_m))
    
    
    
        lres <- rep(0, length(eps))
    
        for(l in 1:length(eps)){
          lres[l] <- log(q) - gam*log(1+a_t*abs(level[l] - b)^c)
        }
        return(list(coef = c(a_t, b, c, q, gam), eps = eps, 
                    level = level))
    
      }
    
    
    
      ## Calculation VaR
      VaR <- function(X, alpha){
        v <- ACER_General(X, alpha)
        coef <- v$coef
        coef <- unlist(coef)
        a_t <- coef[1]
        b <- coef[2]
        c <- coef[3]
        q <- coef[4]
        gam <- coef[5]
        # VaR <- b+abs(1/a_t*abs(alpha/q)^(1/gam) - 1)^(1/c)
        VaR_new <- b + abs(((alpha/q)^(-1/gam)-1)/a_t)^(1/c)
        M <- max(X)
        if(is.na(VaR_new)){
          print("There is a problem with the optimization")
        }
        else{
          if(VaR_new > M){
            print("There is a problem with the optimization")
          }
        }
        return(VaR_new)
      }
    
      ## DATA FRAME ##
      l <- 20
      VaR_E <- rep(0,l)
      VaR_hist <- rep(0,l)
      VaR_R <- rep(0,l)
      VaR_MC <- rep(0,l)
      VaR_ACER <- rep(0,l)
      e_E <- rep(0,l)
      e_hist <- rep(0,l)
      e_R <- rep(0,l)
      e_MC <- rep(0,l)
      e_ACER <- rep(0,l)
    
      for(i in 1:l){
        ## DATA  ##
        N <- 800
        X <- rpareto(N, 3, 1)
        D <- X[2:N] - X[1:(N-1)]
    
    
        ## Riskmetrics ##
        P <- log(X)
        D_R <- P[2:N] - P[1:(N-1)]
        VaR_R[i] <- mean(X)*1.645*sqrt(exp(var(D_R)))
        e_R[i] <- (VaR_R[i]-2.7)*100/2.7
    
        ## Econometric approach ##
        P <- log(X)
        G <- garch(P)
        coeff <- G$coef
        VaR_E[i] <- exp(sum(coeff * P[(N-length(coeff)+1) : N]))
                    + 1.645*exp(var(P))
        e_E[i] <- (VaR_E[i]-2.7)*100/2.7
    
        ## Historical Method  ##
        D_hist <- sort(D)
        X_hist <- mean(X)+D_hist
        VaR_hist[i] <- X_hist[0.95*N]
        e_hist[i] <- (VaR_hist[i]-2.7)*100/2.7
    
        ## MC Method  ##
        D_MC <- D
        Sigm <- var(D_MC)
    
        l_r <- 10000
        r <- runif(l_r, 0, 1)
    
        X_MC <- X[N]  + Sigm * r
        X_MC <- sort(X_MC)
        VaR_MC[i] <- X_MC[0.95*l_r]
        e_MC[i] <- (VaR_MC[i]-2.7)*100/2.7
    
        ## ACER method ##
        VaR_ACER[i] <- VaR(X,0.05)
        e_ACER[i] <- (VaR_ACER[i]-2.7)*100/2.7
    
      }
    
    
      dt <- data.frame(Riskmetric = e_R, Econometric = e_E, 
               Historical = e_hist, MC = e_MC, ACER = e_ACER)
    
      w <- formattable(dt)
      sign_formatter <- formatter("span",
            style = x ~ style(color = ifelse(x > 10, "red",
            ifelse(x < -10 | x == "NaN", "orange", "green"))))
      w <- formattable(dt, list(Riskmetric = sign_formatter, 
           Econometric =sign_formatter, Historical =sign_formatter, 
           MC =sign_formatter, ACER =sign_formatter))
    
      export_formattable <- function(f, file, width = "100%", 
                          height = NULL, 
                          background = "white", delay = 0.2)
      {
        w <- as.htmlwidget(f, width = width, height = height)
        path <- html_print(w, background = background, viewer = NULL)
        url <- paste0("file:///", gsub("\\\\", "/", 
                                       normalizePath(path)))
        webshot(url,
                file = file,
                selector = ".formattable_widget",
                delay = delay)
      }
      export_formattable(w, 
      "C:/Users/Mala/Desktop/Master's thesis/Latex/fig/w2.png")
}


#### test soge ####

#This function f_k will count the number of different values
# of the ACER functions between eta_1 and eta_f
f_k<- function(X,k,eta1, etaf, alpha)
{
  
  # aux <- CI_boot(X,mean(X),k, 50, alpha)
  aux <- CI_Poisson(X, mean(X),k)
  aux1 <- aux$epsilon
  aux_eps <- aux1
  nb_points <- 100
  pas <- abs(etaf - eta1)/(nb_points)
  if(eta1 >= etaf){
    etaf <- 2*eta1
  }
  test_level <- seq(eta1, etaf, pas)
  eps <- rep(0,nb_points)
  for(level in test_level)
  {
    # aux <- CI_boot(X, level,k, 100)
    aux <- CI_Poisson(X, level,k)
    
    aux1 <- aux$epsilon
    eps <- c(eps, aux1)
  }
  
  v_dif <- length(unique(eps[eps!=0]))
  
  return(v_dif)
}

#This function detect_k will choose a k to
# get a value of k as small as possible without
# having a lack of precision with long sequences
#of constant values
detect_k <- function(X, eta1, etaf, alpha)
{
  N <- length(X)
  M <- 105
  m <- 5
  bi <- f_k(X,m, eta1, etaf, alpha)
  
  q <- floor(0.8*bi)
  while((M-m) > 5){
    m_aux <- floor((M+m)/2)
    v <- f_k(X,m_aux, eta1, etaf, alpha)
    if(v < q){
      M <- m_aux
    }
    else{
      m <- m_aux
    }
  }
  return(m)
}



## Calculation of etaf1, eta_f
#eta1 will represent the lowest level and eta_f the
#highest of our study
eta_detect <- function(X, k, alpha)
{
  nb_values <- 200
  pas <- (max(X)-min(X))/nb_values
  test_level <- seq(min(X),max(X),pas)
  i <- 1
  k_i <- k
  aux_end <- rep(0,k_i)
  for(level in test_level)
  {
    # aux <- CI_boot(X,level,k_i,100, alpha)
    aux <- CI_Poisson(X, level, k_i)
    aux1 <- aux$epsilon
    if(aux1>0){
      aux_end[i] <- (aux1)
      
    }
    else{
      aux_end[i] <- 0
    }
    i = i+1
  }
  m <- max(aux_end)
  ind <- max(which(aux_end == m))
  v <- aux_end[ind:length(aux_end)]
  mu1 <- test_level[ind]
  w <- diff(log(v))
  l <- which(w != 0)
  r <- diff(l)
  
  m <- min(which(r >8))
  p <- l[m]
  muf <- test_level[p]
  if(muf <= mu1){
    muf <- 2*mu1
  }
  return(c(mu1, muf))
}

## ACER method
ACER_General <- function(X, alpha)
{
  #First we fixe k at a possible value
  # and we find eta1 and eta_f. These two values
  #do not vary too much with k
  eta <- eta_detect(X, 3, alpha)
  eta1 <- eta[1]
  etaf <- eta[2]
  
  #With eta1 and eta_f, we find k.
  k <- detect_k(X, eta1, etaf, alpha)
  
  #We calculate then the true values of eta1 and eta_f
  eta <- eta_detect(X, k, alpha)
  eta1 <- eta[1]
  etaf <- eta[2]


  nb_points <- 200
  if(eta1 == etaf){
    etaf <- 2*eta1
  }
  pas <- (etaf-eta1)/(nb_points)
  test_level <- seq(eta1, etaf, pas)
  aux_eps <- 0
  aux_w <- 0
  for(level in test_level)
  {
    # aux <- CI_boot(X, level,k, 100)
    aux <- CI_Poisson(X, level, k)
    aux1 <- aux$epsilon
    aux2 <- unlist(aux$CI)
    v <- 1/(log(abs(aux2[1])) -log(abs(aux2[2])))^2
    
    aux_eps <- c(aux_eps, aux1)
    aux_w <- c(aux_w, v)
    
  }
  
  eps <- aux_eps[2:length(aux_eps)]
  w <- aux_w[2:length(aux_w)]
  
  level <- test_level
  
  y <- log(eps)
  y_m <- sum(w*y) / sum(w)
  
  ACER_sim_vect <- function(param){
    #we are creating now the function for the given k
    b <- param[2]
    c <- param[3]
    a_t <- param[1]
    x <- log(abs(1 + a_t*abs(level - b)^c))
    x_m <- sum(w*x)/sum(w)
    gam <- -(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
    q <- exp(y_m + gam*x_m)
    result <- rep(0, length(w))
    for(i in 1:length(w)){
      result[i] <- as.double(sqrt(w[i])*abs(y[i] - log(q)
                    + gam*log(abs(1 + a_t*abs(level[i] - b)^c))))
    }
    return(unlist(result))
  }
  #
  #
  m <- min(X)
  # we need to find b, c
  minit <- (m+eta1)/2
  
  
  o <- nls.lm(c(1,minit, 1), lower = c(0,m, 0), 
              upper = c(Inf,eta1, 5), ACER_sim_vect,  
              jac = NULL, 
              nls.lm.control(ftol = 0.001, maxiter = 1000))
  
  
  param <- o$par
  #Now, we need to find a, q_ln
  a_t <- param[1]
  b <- param[2]
  c <- param[3]
  x <- log((1 + a_t*abs(level - b)^c))
  x_m <- sum(w*x)/sum(w)
  
  gam <- -(sum(w*(x-x_m)*(y-y_m))/sum((w*(x-x_m)^2)))
  
  q <- exp((y_m + gam*x_m))
  
  
  
  lres <- rep(0, length(eps))
  
  for(l in 1:length(eps)){
    lres[l] <- log(q) - gam*log(1+a_t*abs(level[l] - b)^c)
  }
  return(list(coef = c(a_t, b, c, q, gam), eps = eps, 
              level = level))
  
}

## Calculation VaR
VaR <- function(X, alpha){
  v <- ACER_General(X, alpha)
  coef <- v$coef
  coef <- unlist(coef)
  a_t <- coef[1]
  b <- coef[2]
  c <- coef[3]
  q <- coef[4]
  gam <- coef[5]
  # VaR <- b+abs(1/a_t*abs(alpha/q)^(1/gam) - 1)^(1/c)
  VaR_new <- b + abs(((alpha/q)^(-1/gam)-1)/a_t)^(1/c)
  M <- max(X)
  if(is.na(VaR_new)){
    print("There is a problem with the optimization")
  }
  else{
    if(VaR_new > M){
      print("There is a problem with the optimization")
    }
  }
  return(VaR_new)
}

### Every day
options(digits = 6)
data<-read.table(
  "C:/Users/Mala/Desktop/Master's thesis/R/final/quotes.csv",
  sep=";") 
op_a <- levels(data$V2)
op <- op_a[3:(length(op_a)-5)]
op <- sub(",", ".", op)
op <- as.numeric(op)
soge_test_everyday <- function(op){
  D <- diff(op)
  pts <- 1000
  nb <- 30
  frc <- rep(0,nb)
  start <- 1100
  frc[1] <- op[start+1]
  for(i in 2:nb){
    u <- VaR(D[(start+i-pts):(start+i)], 0.05)
    D[start+i+1] <- u
    frc[i] <- frc[i-1] + u
  }
  plot(op[(start+1):(start+nb)], ylim = c(44.6, 48), 
       ylab = "Values of the opening", xlab = "Days")
  lines(frc, col = "red")
}
    
soge_test_week <- function(op){
  aux <- seq(5,1405,5) #281 values
  D <- diff(op[aux])
  nb <- 7
  frc <- rep(0,nb)
  frc[1] <- op[1000]
  for(i in 2:nb){
    u <- VaR(D[i:(150+i)], 0.05)
    D[start+i+1] <- u
    frc[i] <- frc[i-1] + u
  }
  aux <- seq(1000, 1000+(nb-1)*5,5)
  plot(x = aux, op[aux], ylim = c(43,45),
       ylab = "Values of the opening", xlab = "Days")
  lines(aux,frc, col = "red")

}